/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Paste and Go 2.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Holio Lin <gaghiel@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var MenuRoll = {
    menuShowing: false,
    currentItemIndex: 0,
    searchbar: null,
    popup: null,
    //engines: null, // dynamic
    items: null,
    rollKey: null,
    delayMS: 0,

    startup: function(searchbar, rollKey, delayMS) {
        MenuRoll.searchbar = searchbar;
        MenuRoll.popup = searchbar._popup;
        MenuRoll.items = searchbar._popup.childNodes;
        //MenuRoll.engines = searchbar._engines;
        MenuRoll.rollKey = rollKey;

        window.addEventListener("keypress", MenuRoll.onceKeyPress, true);
        window.addEventListener("keyup", MenuRoll.onceKeyUp, true);

        MenuRoll.popup.addEventListener("popuphidden", MenuRoll.onceMenuHidden, false);

        MenuRoll.popup.addEventListener("DOMMenuItemActive", MenuRoll.ItemActive, true);
        MenuRoll.popup.addEventListener("DOMMenuItemInactive", MenuRoll.ItemInactive, true);

        MenuRoll.delayMS = delayMS;
    },

    shutdown: function() {
        window.removeEventListener("keypress", MenuRoll.onceKeyPress, true);
        window.removeEventListener("keyup", MenuRoll.onceKeyUp, true);
    },

    getEngines: function() { // dynamic
        return MenuRoll.searchbar._engines;
    },

    ItemActive: function(event) {
        var currentItem = MenuRoll.items[MenuRoll.currentItemIndex];
        if (MenuRoll.menuShowing && event.target != currentItem)
            event.target.setAttribute("_moz-menuactive", "false");
    },

    ItemInactive: function(event) {
        var currentItem = MenuRoll.items[MenuRoll.currentItemIndex];
        if (MenuRoll.menuShowing && event.target == currentItem)
            event.target.setAttribute("_moz-menuactive", "true");
    },

    onceKeyPress: function(event) {
        
        if ((String.fromCharCode(event.charCode).toUpperCase() == MenuRoll.rollKey.toUpperCase()) && event.ctrlKey && event.shiftKey && !event.altKey) {

            if (MenuRoll.menuShowing) {
                MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "false");
                MenuRoll.currentItemIndex = (MenuRoll.currentItemIndex + 1) % MenuRoll.getEngines().length;
            } else {
                MenuRoll.menuShowing = true;
                
                if (MenuRoll.delayMS == 0)
                    MenuRoll.popup.click();
                else {
                    window.setTimeout( function() {
                        if (MenuRoll.menuShowing)   // synchronization
                            MenuRoll.popup.click(); // show
                    }, MenuRoll.delayMS);
                }
            }

            MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "true");

            event.stopPropagation();
            event.preventDefault();
        }
    },

    onceMenuHidden: function(event) { // reset state
        MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "false");
        MenuRoll.currentItemIndex = 0;
        MenuRoll.menuShowing = false;
    },

    onceKeyUp: function(event) {
        if (MenuRoll.menuShowing && event.keyCode == event.DOM_VK_CONTROL) {// && !event.altKey && !event.metaKey)) {
            MenuRoll.menuShowing = false; // avoid synchronization failedd
            searchByEngine(MenuRoll.getEngines()[MenuRoll.currentItemIndex]);
            MenuRoll.popup.hidePopup();   // won't invoke onceMenuHidden
            MenuRoll.onceMenuHidden();
        }
    },

} // end of MenuRoll
